/*
    SSSD

    Common helper functions to handle child processes

    Authors:
        Sumit Bose   <sbose@redhat.com>

    Copyright (C) 2009 Red Hat

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __CHILD_COMMON_H__
#define __CHILD_COMMON_H__

#include <errno.h>
#include <sys/types.h>
#include <tevent.h>

#include "shared/io.h"
#include "util/util.h"

/* **********   Child process handling helpers (child_common.c)   ********** */

struct child_io_fds {
    int read_from_child_fd;
    int write_to_child_fd;
    pid_t pid;
    bool child_exited;
    bool in_use;
    struct tevent_timer *timeout_handler;
};

struct sss_child_ctx;

/* Callback to be invoked when a sigchld handler is called.
 * The tevent_signal * associated with the handler will be
 * freed automatically when this function returns.
 */
typedef void (*sss_child_sigchld_callback_t)(int child_status,
                                             struct tevent_signal *sige,
                                             void *pvt);

typedef void (*sss_child_timeout_callback_t)(struct tevent_context *ev,
                                             struct tevent_timer *te,
                                             struct timeval tv, void *pvt);

errno_t sss_child_start(TALLOC_CTX *mem_ctx,
                        struct tevent_context *ev,
                        const char *binary,
                        const char *extra_args[], bool extra_args_only,
                        const char *logfile,
                        int child_out_fd,  /* FD that binary uses to write response to */
                        sss_child_sigchld_callback_t cb,  /* SIGCHLD handler */
                        void *pvt,  /* SIGCHLD callback context, NULL means `*_io` */
                        unsigned timeout,  /* timeout to invoke timeout_cb, 0 means no timeout */
                        sss_child_timeout_callback_t timeout_cb,
                        void *timeout_ptv,  /* timeout callback context */
                        struct child_io_fds **_io);

/* Set up child termination signal handler */
int child_handler_setup(struct tevent_context *ev, int pid,
                        sss_child_sigchld_callback_t cb, void *pvt,
                        struct sss_child_ctx **_child_ctx);

/* Destroy child termination signal handler */
void child_handler_destroy(struct sss_child_ctx *ctx);

/* Never returns EOK, ether returns an error, or doesn't return on success */
void exec_child_ex(TALLOC_CTX *mem_ctx,
                   int *pipefd_to_child, int *pipefd_from_child,
                   const char *binary, const char *logfile,
                   const char *extra_argv[], bool extra_args_only,
                   int child_in_fd, int child_out_fd);

int child_io_destructor(void *ptr);


void child_exited(int child_status, struct tevent_signal *sige, void *pvt);

void child_terminate(pid_t pid);

/* **************************   IPC (child_io.c)   ************************* */

/* Async communication with the child process via a pipe */
struct tevent_req *write_pipe_send(TALLOC_CTX *mem_ctx,
                                   struct tevent_context *ev,
                                   uint8_t *buf,
                                   size_t len,
                                   int fd);
int write_pipe_recv(struct tevent_req *req);

struct tevent_req *read_pipe_send(TALLOC_CTX *mem_ctx,
                                  struct tevent_context *ev,
                                  int fd);
struct tevent_req *read_pipe_non_blocking_send(TALLOC_CTX *mem_ctx,
                                               struct tevent_context *ev,
                                               int fd);
errno_t read_pipe_recv(struct tevent_req *req,
                       TALLOC_CTX *mem_ctx,
                       uint8_t **_buf,
                       ssize_t *_len);

/* Include buffer length in a message header, read does not wait for EOF. */
struct tevent_req *write_pipe_safe_send(TALLOC_CTX *mem_ctx,
                                        struct tevent_context *ev,
                                        uint8_t *buf,
                                        size_t len,
                                        int fd);
int write_pipe_safe_recv(struct tevent_req *req);

struct tevent_req *read_pipe_safe_send(TALLOC_CTX *mem_ctx,
                                       struct tevent_context *ev,
                                       int fd);
errno_t read_pipe_safe_recv(struct tevent_req *req,
                            TALLOC_CTX *mem_ctx,
                            uint8_t **_buf,
                            ssize_t *_len);

#endif /* __CHILD_COMMON_H__ */
